@@ -26,7 +26,8 @@ alias_by_command: Dict[str, str] = {
     "edit": "e",
     "go": "g",
     "log": "l",
-    "status": "s"
+    "status": "s",
+    "traverse": "t"
 }
 
 command_groups: List[Tuple[str, List[str]]] = [
@@ -316,6 +317,7 @@ def create_cli_parser() -> argparse.ArgumentParser:
 
     traverse_parser = subparsers.add_parser(
         'traverse',
+        aliases=['t'],
         argument_default=argparse.SUPPRESS,
         usage=argparse.SUPPRESS,
         add_help=False,
@@ -332,7 +334,7 @@ def create_cli_parser() -> argparse.ArgumentParser:
     traverse_parser.add_argument('--push-untracked', action='store_true')
     traverse_parser.add_argument('--no-push-untracked', action='store_true')
     traverse_parser.add_argument('--return-to')
-    traverse_parser.add_argument('--start_from')
+    traverse_parser.add_argument('--start-from')
     traverse_parser.add_argument('-w', '--whole', action='store_true')
     traverse_parser.add_argument('-W', action='store_true')
     traverse_parser.add_argument('-y', '--yes', action='store_true')
@@ -748,7 +750,7 @@ def launch(orig_args: List[str]) -> None:
                 opt_list_commits=cli_opts.opt_list_commits,
                 opt_list_commits_with_hashes=cli_opts.opt_list_commits_with_hashes,
                 opt_no_detect_squash_merges=cli_opts.opt_no_detect_squash_merges)
-        elif cmd == "traverse":
+        elif cmd in {"traverse", alias_by_command["traverse"]}:
             if cli_opts.opt_start_from not in {"here", "root", "first-root"}:
                 raise MacheteException(
                     "Invalid argument for `--start-from`. "
@@ -788,13 +790,13 @@ def launch(orig_args: List[str]) -> None:
 
     except (argparse.ArgumentError, argparse.ArgumentTypeError) as e:
         print(get_short_general_usage())
-        sys.stderr.write(f"\n{e}\n")
+        print(f"\n{e}", file=sys.stderr)
         exit_script(2)
     except MacheteException as e:
-        sys.stderr.write(f"\n{e}\n")
+        print(f"\n{e}", file=sys.stderr)
         exit_script(1)
     except KeyboardInterrupt:
-        sys.stderr.write("\nInterrupted by the user")
+        print("\nInterrupted by the user", file=sys.stderr)
         exit_script(1)
     except StopInteraction:
         pass
