@@ -208,8 +208,13 @@ long_docs: Dict[str, str] = {
     "file": """
         <b>Usage: git machete file</b>
 
-        Outputs the absolute path of the machete definition file. Currently fixed to `<git-directory>/machete`.
-        Note: this won't always be just `<repo-root>/.git/machete` since e.g. submodules and worktrees have their git directories in different location.
+        Outputs the absolute path of machete definition file.
+        The file is always called `machete` and is located in the git directory of the project.
+
+        Three cases are possible:
+        * if `git machete` is executed from a regular working directory (not a worktree or submodule), the file is located under `.git/machete`,
+        * if `git machete` is executed from a <b>worktree</b>, this file is located under `.git/machete` as well (<b>not</b> in the git folder of the worktree under `.git/worktrees/.../machete`),
+        * if `git machete` is executed from a <b>submodule</b>, this file is located in the git folder of the submodule itself under `.git/modules/.../machete`.
     """,
     "fork-point": """
         <b>Usage:
@@ -606,10 +611,10 @@ long_docs: Dict[str, str] = {
                                             when detecting if a branch is merged into its upstream (parent).
     """,
     "traverse": """
-        <b>Usage: git machete traverse [-F|--fetch] [-l|--list-commits] [-M|--merge]
-                                       [-n|--no-edit-merge|--no-interactive-rebase] [--no-detect-squash-merges]
-                                       [--[no-]push] [--[no-]push-untracked]
-                                       [--return-to=WHERE] [--start-from=WHERE] [-w|--whole] [-W] [-y|--yes]</b>
+        <b>Usage: git machete t[raverse] [-F|--fetch] [-l|--list-commits] [-M|--merge]
+                                         [-n|--no-edit-merge|--no-interactive-rebase] [--no-detect-squash-merges]
+                                         [--[no-]push] [--[no-]push-untracked]
+                                         [--return-to=WHERE] [--start-from=WHERE] [-w|--whole] [-W] [-y|--yes]</b>
 
         Traverses the branch dependency tree in pre-order (i.e. simply in the order as they occur in the definition file).
         By default traverse starts from current branch, however this behaviour can be customized using options: `--starts-from=`, `--whole` or `-w`, `-W`.
