@@ -1,3 +1,4 @@
+from pathlib import Path
 from typing import Callable, Dict, Generator, Iterator, List, Match, Optional, Set, Tuple
 
 import os
@@ -126,10 +127,10 @@ class ShortCommitHash(AnyRevision):
     @staticmethod
     def of(value: str) -> Optional["ShortCommitHash"]:
         if value:
-            if len(value) == 7:
+            if len(value) >= 7:
                 return ShortCommitHash(value)
             else:
-                raise TypeError(f'ShortCommitHash requires length of 7. Given value: {value}, has length: {len(value)}.\n'
+                raise TypeError(f'ShortCommitHash requires length greater or equal to 7. Given value: {value}, has length: {len(value)}.\n'
                                 'Consider posting an issue on https://github.com/VirtusLab/git-machete/issues/new')
         return None
 
@@ -143,8 +144,15 @@ class FullTreeHash(str):
         return FullTreeHash(value) if value else None
 
 
+class ForkPointOverrideData:
+    def __init__(self, to_hash: FullCommitHash, while_descendant_of_hash: FullCommitHash):
+        self.to_hash: FullCommitHash = to_hash
+        self.while_descendant_of_hash: FullCommitHash = while_descendant_of_hash
+
+
 ReflogEntry = Tuple[FullCommitHash, str]
 
+
 HEAD = AnyRevision.of("HEAD")
 
 
@@ -164,8 +172,8 @@ class GitContext:
         self.__committer_unix_timestamp_by_revision_cached: Optional[Dict[AnyRevision, int]] = None  # TODO (#110): default dict with 0
         self.__local_branches_cached: Optional[List[LocalBranchShortName]] = None
         self.__remote_branches_cached: Optional[List[RemoteBranchShortName]] = None
-        self.__initial_log_shas_cached: Dict[LocalBranchShortName, List[FullCommitHash]] = {}
-        self.__remaining_log_shas_cached: Dict[LocalBranchShortName, List[FullCommitHash]] = {}
+        self.__initial_log_shas_cached: Dict[FullCommitHash, List[FullCommitHash]] = {}
+        self.__remaining_log_shas_cached: Dict[FullCommitHash, List[FullCommitHash]] = {}
         self.__reflogs_cached: Optional[Dict[AnyBranchName, Optional[List[ReflogEntry]]]] = None
         self.__merge_base_cached: Dict[Tuple[FullCommitHash, FullCommitHash], FullCommitHash] = {}
         self.__contains_equivalent_tree_cached: Dict[Tuple[FullCommitHash, FullCommitHash], bool] = {}
@@ -219,10 +227,9 @@ class GitContext:
                     debug("get_default_editor()", f"{editor_repr} is available")
                     if name != "$" + git_machete_editor_var and self.get_config_attr_or_none('advice.macheteEditorSelection') != 'false':
                         sample_alternative = 'nano' if editor.startswith('vi') else 'vi'
-                        sys.stderr.write(
-                            fmt(f"Opening <b>{editor_repr}</b>.\n",
-                                f"To override this choice, use <b>{git_machete_editor_var}</b> env var, e.g. `export {git_machete_editor_var}={sample_alternative}`.\n\n",
-                                "See `git machete help edit` and `git machete edit --debug` for more details.\n\nUse `git config --global advice.macheteEditorSelection false` to suppress this message.\n"))
+                        print(fmt(f"Opening <b>{editor_repr}</b>.\n",
+                                  f"To override this choice, use <b>{git_machete_editor_var}</b> env var, e.g. `export {git_machete_editor_var}={sample_alternative}`.\n\n",
+                                  "See `git machete help edit` and `git machete edit --debug` for more details.\n\nUse `git config --global advice.macheteEditorSelection false` to suppress this message."), file=sys.stderr)
                     return editor
 
         # This case is extremely unlikely on a modern Unix-like system.
@@ -248,7 +255,14 @@ class GitContext:
     def __get_git_dir(self) -> str:
         if not self._git_dir:
             try:
-                self._git_dir = self._popen_git("rev-parse", "--git-dir").strip()
+                git_dir: str = self._popen_git("rev-parse", "--git-dir").strip()
+                git_dir_parts = Path(git_dir).parts
+                if len(git_dir_parts) >= 3 and git_dir_parts[-3] == '.git' and git_dir_parts[-2] == 'worktrees':
+                    self._git_dir = os.path.join(*git_dir_parts[:-2])
+                    debug('__get_git_dir', f'git dir pointing to {git_dir} - we are in a worktree; '
+                                           f'using {self._git_dir} as the effective git dir instead')
+                else:
+                    self._git_dir = git_dir
             except MacheteException:
                 raise MacheteException("Not a git repository")
         return self._git_dir
@@ -355,6 +369,8 @@ class GitContext:
             return None
 
     def get_commit_sha_by_revision(self, revision: AnyRevision) -> Optional[FullCommitHash]:
+        if self.is_full_sha(revision.full_name()):
+            return FullCommitHash.of(revision)
         if self.__commit_sha_by_revision_cached is None:
             self.__load_branches()
         if revision not in self.__commit_sha_by_revision_cached:
@@ -378,13 +394,6 @@ class GitContext:
     def is_full_sha(revision: AnyRevision) -> Optional[Match[str]]:
         return re.match("^[0-9a-f]{40}$", revision)
 
-    # Resolve a revision identifier to a full sha
-    def get_full_sha(self, revision: AnyRevision) -> Optional[FullCommitHash]:
-        if self.is_full_sha(revision.full_name()):
-            return FullCommitHash.of(revision)
-        else:
-            return self.get_commit_sha_by_revision(revision)
-
     def get_committer_unix_timestamp_by_revision(self, revision: AnyBranchName) -> int:
         if self.__committer_unix_timestamp_by_revision_cached is None:
             self.__load_branches()
@@ -490,15 +499,15 @@ class GitContext:
     # Since getting the full history of a branch can be an expensive operation for large repositories (compared to all other underlying git operations),
     # there's a simple optimization in place: we first fetch only a couple of first commits in the history,
     # and only fetch the rest if needed.
-    def spoonfeed_log_shas(self, branch: LocalBranchShortName) -> Generator[FullCommitHash, None, None]:
-        if branch not in self.__initial_log_shas_cached:
-            self.__initial_log_shas_cached[branch] = self.__get_log_shas(branch, max_count=MAX_COUNT_FOR_INITIAL_LOG)
-        for sha in self.__initial_log_shas_cached[branch]:
+    def spoonfeed_log_shas(self, branch_full_hash: FullCommitHash) -> Generator[FullCommitHash, None, None]:
+        if branch_full_hash not in self.__initial_log_shas_cached:
+            self.__initial_log_shas_cached[branch_full_hash] = self.__get_log_shas(branch_full_hash, max_count=MAX_COUNT_FOR_INITIAL_LOG)
+        for sha in self.__initial_log_shas_cached[branch_full_hash]:
             yield FullCommitHash.of(sha)
 
-        if branch not in self.__remaining_log_shas_cached:
-            self.__remaining_log_shas_cached[branch] = self.__get_log_shas(branch, max_count=None)[MAX_COUNT_FOR_INITIAL_LOG:]
-        for sha in self.__remaining_log_shas_cached[branch]:
+        if branch_full_hash not in self.__remaining_log_shas_cached:
+            self.__remaining_log_shas_cached[branch_full_hash] = self.__get_log_shas(branch_full_hash, max_count=None)[MAX_COUNT_FOR_INITIAL_LOG:]
+        for sha in self.__remaining_log_shas_cached[branch_full_hash]:
             yield FullCommitHash.of(sha)
 
     def __load_all_reflogs(self) -> None:
@@ -552,16 +561,11 @@ class GitContext:
         self.__remotes_cached = None
         self.__counterparts_for_fetching_cached = None
         self.__short_commit_sha_by_revision_cached = {}
-        self.__tree_sha_by_commit_sha_cached = None
         self.__commit_sha_by_revision_cached = None
         self.__committer_unix_timestamp_by_revision_cached = None
         self.__local_branches_cached = None
         self.__remote_branches_cached = None
-        self.__initial_log_shas_cached = {}
-        self.__remaining_log_shas_cached = {}
         self.__reflogs_cached = None
-        self.__merge_base_cached = {}
-        self.__contains_equivalent_tree_cached = {}
 
     def get_revision_repr(self, revision: AnyRevision) -> str:
         short_sha = self.get_short_commit_sha_by_revision(revision)
@@ -651,8 +655,8 @@ class GitContext:
             earlier_revision: AnyRevision,
             later_revision: AnyRevision,
     ) -> bool:
-        earlier_sha = self.get_full_sha(earlier_revision)
-        later_sha = self.get_full_sha(later_revision)
+        earlier_sha = self.get_commit_sha_by_revision(earlier_revision)
+        later_sha = self.get_commit_sha_by_revision(later_revision)
         # This if statement is not changing the outcome of the later return, but
         # it enhances the efficiency of the script. If both hashes are the same,
         # there is no point running git merge-base.
@@ -669,8 +673,8 @@ class GitContext:
             earlier_revision: AnyRevision,
             later_revision: AnyRevision,
     ) -> bool:
-        earlier_commit_sha = self.get_full_sha(earlier_revision)
-        later_commit_sha = self.get_full_sha(later_revision)
+        earlier_commit_sha = self.get_commit_sha_by_revision(earlier_revision)
+        later_commit_sha = self.get_commit_sha_by_revision(later_revision)
 
         if earlier_commit_sha == later_commit_sha:
             return True
@@ -730,10 +734,8 @@ class GitContext:
             advice_ignored_hook = self.get_config_attr_or_none("advice.ignoredHook")
             if advice_ignored_hook != 'false':  # both empty and "true" is okay
                 # The [33m color must be used to keep consistent with how git colors this advice for its built-in hooks.
-                sys.stderr.write(
-                    colored(f"hint: The '{hook_path}' hook was ignored because it's not set as executable.", EscapeCodes.YELLOW) + "\n")
-                sys.stderr.write(
-                    colored("hint: You can disable this warning with `git config advice.ignoredHook false`.", EscapeCodes.YELLOW) + "\n")
+                print(colored(f"hint: The '{hook_path}' hook was ignored because it's not set as executable.", EscapeCodes.YELLOW), file=sys.stderr)
+                print(colored("hint: You can disable this warning with `git config advice.ignoredHook false`.", EscapeCodes.YELLOW), file=sys.stderr)
             return False
         else:
             return True
